@@ -16,7 +16,7 @@
 ##    along with Fonty Python.  If not, see <http://www.gnu.org/licenses/>.
 
 
-import sys, codecs, locale, os
+import sys, locale, os
 import strings
 import pathcontrol
 import fpsys
@@ -40,63 +40,88 @@ class options(object):
     purge = False
     install = None
     uninstall = None
-
+    check = False
+    
 ## If non-ascii chars get entered on the cli, say a Japanese word for
 ## a pog's name, we may have a problem. 
-## Our app relies on all input being UNICODE.
-## So, I am going to decode those byte strings into Unicode first:    
+## So, I am going to (try to) decode those byte strings into Unicode first:    
 tmp = []
 for a in sys.argv[1:]:
-    if type(a) != type(u""):
-        a = a.decode( locale.getpreferredencoding() ) # Wild stab in the dark here...
+    ## It seems that a is always a BYTE STRING, but I'll just test anyway:
+    #print [a]
+    if type(a) is str:
+        ## This happens when text is PASTED onto the cli when that cli
+        ## is in a LANG that can't handle that text.
+        ## I don't know what other cases may cause this.
+        try:
+            a = a.decode( locale.getpreferredencoding() )
+        except:
+            print _(u"I can't decode your argument(s). Please check your LANG variable. Also, don't paste text, type it in.")
+            raise SystemExit
     tmp.append(a)
 uargs = tmp
 
 try:
-    opts, args = getopt.gnu_getopt(uargs, "hveli:u:s:n:t:p:", ["help", "version"])
+    opts, args = getopt.gnu_getopt(uargs, "hvlc:ei:u:s:n:t:p:",\
+    ["help", "version", "list", "check=","examples","install=",\
+    "uninstall=","size=","number=","text=","purge="])
 except getopt.GetoptError, err:
     print str(err) # will print something like "option -a not recognized"
     raise SystemExit
 
 for o, a in opts:
+    if o in (u"-c", u"--check"):
+        dirtocheck = os.path.abspath( a )
+        options.check = True
+        break
     if o in ("-v", "--version"):
         print strings.version
         raise SystemExit
-    if o == "-e":
+
+    if o in ("-e", "--examples"):
         print strings.examples
         raise SystemExit
+        
     elif o in ("-h", "--help"):
         print strings.use
         print strings.options
         raise SystemExit
-    elif o == "-l":
+        
+    elif o in ("-l", "--list"):
         options.list = True
-    elif o == "-i":
+
+    elif o in ("-i","--install"):
         options.install = a
-    elif o == "-u":
+
+    elif o in ("-u", "--uninstall"):
         options.uninstall = a
-    elif o == "-p":
+
+    elif o in ("-p", "--purge"):
         options.purge = a
-    elif o == "-s":
+
+    elif o in ("-s", "--size"):
         try:
             n = int(a)
         except:
             print _("Please use a number for %s") % o
             raise SystemExit 
         options.points = n
-    elif o == "-n":
+
+    elif o in ("-n", "--number"):
         try:
             n = int(a)
         except:
             print _("Please use a number for %s") % o
             raise SystemExit            
         options.numinpage = n
-    elif o == "-t":
+
+    elif o in ("-t", "--text"):
         ## The quotes gets stripped out, so I can't tell the
         ## end of the text param and the start of other params
         ## therefore I stick to the 'report and restart' idea.
         options.text = a
         fpsys.config.text = a
+        
     else:
         ## We should not reach here at all.
         print "Your arguments amuse me :) Please read the help."
@@ -108,7 +133,24 @@ iPC = pathcontrol.PathControl()
 
 
 ####
-## Let's handle those options that DO NOT require args
+## Let's handle those options that DO NOT require args.
+
+## Check fonts
+if options.check:
+    if not os.path.exists( dirtocheck ):
+        print _("I can't find %s") % dirtocheck
+        raise SystemExit    
+        
+    def printer( pstr = "" ):
+        """A func to print strings to cli, called back from checkFonts."""
+        if type(pstr) is str:
+            pstr = unicode(pstr,locale.getpreferredencoding(),errors="replace")
+        print pstr
+        
+    fpsys.checkFonts( dirtocheck, printer )
+    raise SystemExit
+
+
 ## List -  Quick and dirty. 
 if options.list:
     poglist = iPC.getPogNames()
@@ -120,7 +162,7 @@ if options.list:
     for pog in poglist:
         paf = iPC.appPath() + pog + ".pog"
         try:
-            f = codecs.open(paf, "r", sys.getfilesystemencoding() )
+            f = open(paf, "r" ) #  It's a plain byte-string ascii file.
             installed = f.readline()[:-1] #Strips the \n off the end
             f.close()
         except:
@@ -307,8 +349,8 @@ if A and not B:
         try:
             fpsys.instantiateViewPog(A)# creates state.targetobject globally
         except fontybugs.PogInvalid, e:
-            print "ERROR:", unicode(e)
-            sys.exit(unicode( e ))
+            print repr( e )
+            sys.exit()
     ## Because we are catering for a potential full gui,
     ## we must make an official "targetobject" set to None
     fpsys.SetTargetPogToNone()
@@ -320,7 +362,8 @@ if A and B:
         try:
             fpsys.instantiateViewFolder(A)
         except fontybugs.FolderHasNoFonts, e:
-            sys.exit(unicode( e ))
+            ## Let it continue
+            fpsys.config.lastdir = os.path.abspath(A)
         try:
             installed = fpsys.instantiateTargetPog(B)
         except fontybugs.PogInvalid, e:
