@@ -25,8 +25,16 @@ import wx.html as html
 ## Setup wxPython to access translations : enables the stock buttons.
 langid = wx.LANGUAGE_DEFAULT # Picks this up from $LANG
 mylocale = wx.Locale( langid )
-langcode = locale.getlocale()[0][:2]
 
+## langcode = locale.getlocale()[0] # I must not use getlocale...
+## This is suggested by Martin:
+loc = locale.setlocale(locale.LC_CTYPE) # use *one* of the categories (not LC_ALL)
+## returns something like 'en_ZA.UTF-8'
+if loc is None or len(loc) < 2:
+    langcode = 'en'
+else:
+    langcode = loc[:2].lower()# This is going to cause grief in the future...
+ 
 class DialogHelp(wx.Dialog):
     def __init__(self, *args, **kwds):
         kwds["style"] = wx.DEFAULT_DIALOG_STYLE
@@ -42,6 +50,7 @@ class DialogHelp(wx.Dialog):
         dbox.SetSizeHints(self) #made it variable size, stops at 'size' as a min.
         #dbox.Fit(self)
         #self.Layout()
+        win.SetFocus()# magically enables ESC key too! Go figure :)
 class TestHtmlPanel(wx.Panel):
     def __init__(self, parent, size):
         wx.Panel.__init__(self, parent, -1, style=wx.NO_FULL_REPAINT_ON_RESIZE)
@@ -74,8 +83,11 @@ class DialogAbout(wx.Dialog):
         kwds["style"] = wx.DEFAULT_DIALOG_STYLE
         wx.Dialog.__init__(self, *args, **kwds)
         self.nb = wx.Notebook(self, -1, style=0)
+        
         self.notebook_1_pane_2 = wx.Panel(self.nb, -1)
         self.notebook_1_pane_1 = wx.Panel(self.nb, -1)
+        self.notebook_1_pane_3 = wx.Panel(self.nb, -1)
+        
         self.bLOGO = wx.StaticBitmap\
         (self.notebook_1_pane_1, -1, wx.Bitmap(fpsys.mythingsdir + 'aboutfplogo.png', wx.BITMAP_TYPE_ANY))
         
@@ -85,9 +97,12 @@ class DialogAbout(wx.Dialog):
         self.emaillink = wx.TextCtrl\
         (self.notebook_1_pane_1, -1, strings.contact, size =(200,-1 ), style = wx.TE_READONLY)        
         
-        self.REPLACE_ME_WITH_A_TEXTCTRL = wx.TextCtrl\
+        self.GPL_TEXT = wx.TextCtrl\
         (self.notebook_1_pane_2, -1, strings.GPL, style=wx.TE_MULTILINE|wx.TE_READONLY)
  
+        self.THANKS = wx.TextCtrl\
+        (self.notebook_1_pane_3, -1, strings.thanks, style=wx.TE_MULTILINE|wx.TE_READONLY)
+ 
         ID_ESC = 1001
         self.accel = wx.AcceleratorTable([(wx.ACCEL_NORMAL, wx.WXK_ESCAPE, ID_ESC)])
         self.SetAcceleratorTable(self.accel)
@@ -111,6 +126,7 @@ class DialogAbout(wx.Dialog):
         # begin wxGlade: MyDialog.__do_layout
         sizer_1 = wx.BoxSizer(wx.VERTICAL)
         sizer_3 = wx.BoxSizer(wx.HORIZONTAL)
+        sizer_thanks = wx.BoxSizer( wx.HORIZONTAL )
         
         sizerPane1 = wx.BoxSizer(wx.HORIZONTAL)
         sizerPane1.Add(self.bLOGO, 0, 0, 0)
@@ -123,17 +139,26 @@ class DialogAbout(wx.Dialog):
         sizerPane1.Add(textsizer, 1, wx.ALIGN_BOTTOM, 0)
         
         self.notebook_1_pane_1.SetSizer(sizerPane1)
-        
         sizerPane1.Fit(self.notebook_1_pane_1)
         sizerPane1.SetSizeHints(self.notebook_1_pane_1)
         
-        sizer_3.Add(self.REPLACE_ME_WITH_A_TEXTCTRL,1, wx.EXPAND, 0)
+        sizer_3.Add(self.GPL_TEXT,1, wx.EXPAND, 0)
 
         self.notebook_1_pane_2.SetSizer(sizer_3)
         sizer_3.Fit(self.notebook_1_pane_2)
         sizer_3.SetSizeHints(self.notebook_1_pane_2)
+        
+        ## THANKS
+        sizer_thanks.Add( self.THANKS,1, wx.EXPAND,0 )
+        self.notebook_1_pane_3.SetSizer( sizer_thanks )
+        sizer_thanks.Fit( self.notebook_1_pane_3 )
+        sizer_thanks.SetSizeHints( self.notebook_1_pane_3 )
+        
+        
         self.nb.AddPage(self.notebook_1_pane_1, _("About"))
+        self.nb.AddPage(self.notebook_1_pane_3, _("Thanks"))
         self.nb.AddPage(self.notebook_1_pane_2, _("Licence"))
+        
         sizer_1.Add(self.nb, 1, wx.EXPAND, 0)
 
         self.SetSizer(sizer_1)
@@ -148,7 +173,7 @@ class DialogSettings(wx.Dialog):
     def __init__(self, parent):
         #-1 , "Settings", size = wx.DefaultSize, , 
         #     style = wx.DEFAULT_DIALOG_STYLE):
-        wx.Dialog.__init__(self, parent, -1, _("Settings"), pos = wx.DefaultPosition, size =(450,-1))
+        wx.Dialog.__init__(self, parent, -1, _("Settings"), pos = wx.DefaultPosition)#, size =(450,-1))
         
         ## The layout begins:
         font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.FONTWEIGHT_BOLD)
@@ -156,7 +181,7 @@ class DialogSettings(wx.Dialog):
         self.labelHeading.SetFont(font)
 
         self.label_1 = wx.StaticText(self, -1, _("Sample text:"))
-        self.inputSampleString = wx.TextCtrl(self, -1, fpsys.config.text) #, size = (200, -1)) 
+        self.inputSampleString = wx.TextCtrl(self, -1, fpsys.config.text, size = (200, -1)) 
         self.inputSampleString.SetFocus()
         
         self.label_2 = wx.StaticText(self, -1, _("Point size:"))
@@ -172,19 +197,21 @@ class DialogSettings(wx.Dialog):
         box = wx.BoxSizer(wx.HORIZONTAL)
         verticalSizer = wx.BoxSizer(wx.VERTICAL)
         box.Add(verticalSizer, 1, flag = wx.ALL, border = 10)
-        gridSizer = wx.GridSizer(3, 2, 10, 10) 
         
-        verticalSizer.Add(self.labelHeading, 0, 0, 0)
+        gridSizer = wx.FlexGridSizer( rows=3, cols=2, hgap=5, vgap=8 )
         
-        gridSizer.Add(self.label_1, wx.ALIGN_RIGHT, 0, 0)
-        gridSizer.Add(self.inputSampleString, 1, wx.EXPAND, 0)
-        gridSizer.Add(self.label_2, 0, 0, 0)
-        gridSizer.Add(self.inputPointSize, 0, 0, 0)
-        gridSizer.Add(self.label_3, 0, 0, 0)
-        gridSizer.Add(self.inputPageLen, 0, 0, 0)
+        verticalSizer.Add(self.labelHeading, 0, wx.ALIGN_LEFT )
+        verticalSizer.Add( (0,5), 0 )
         
-        verticalSizer.Add(gridSizer, 1, wx.EXPAND, 0)
-
+        gridSizer.Add(self.label_1, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL )
+        gridSizer.Add(self.inputSampleString, 1, wx.EXPAND )
+        gridSizer.Add(self.label_2, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL )
+        gridSizer.Add(self.inputPointSize, 0 )
+        gridSizer.Add(self.label_3, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL )
+        gridSizer.Add(self.inputPageLen, 0 )
+        
+        verticalSizer.Add(gridSizer, 1, wx.EXPAND )
+        verticalSizer.Add((0,10),0) #space between bottom of grid and buttons
         btnsizer = wx.StdDialogButtonSizer()
         
         btn = wx.Button(self, wx.ID_OK)
@@ -195,11 +222,10 @@ class DialogSettings(wx.Dialog):
         btnsizer.AddButton(btn)
         btnsizer.Realize()
         
-        verticalSizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL| wx.ALL, border = 5)
+        verticalSizer.Add(btnsizer, 0, wx.ALIGN_BOTTOM | wx.ALIGN_CENTER_HORIZONTAL, border = 5)
         
         self.SetSizer(box)
-        #sbox.Fit(self) #Window fits to the box.
-        #box.SetSizeHints(self)
+        box.Fit(self) # This triggers the sizers to do their thing.
         self.Layout()
 
 
@@ -210,9 +236,7 @@ class SegfaultDialog(wx.Dialog):
     can tell the user that there was a segfault and why.
     """
     def __init__(self, parent, culprit):
-        #-1 , "Settings", size = wx.DefaultSize, , 
-        #     style = wx.DEFAULT_DIALOG_STYLE):
-        wx.Dialog.__init__(self, parent, -1, _("Oh boy..."), pos = wx.DefaultPosition, size =(450,-1))
+        wx.Dialog.__init__(self, parent, -1, _("Oh boy..."), pos = wx.DefaultPosition )
         
         ## The layout begins:
         font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.FONTWEIGHT_BOLD)
@@ -244,9 +268,62 @@ class SegfaultDialog(wx.Dialog):
         btnsizer.AddButton(btn)
         btnsizer.Realize()
                 
-        verticalSizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL| wx.ALL, border = 15)
+        verticalSizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL| wx.ALIGN_RIGHT, border = 15)
         
         self.SetSizer(box)
+        box.Fit(self)
         self.Layout()
         
+
+class DialogCheckFonts( wx.Dialog ):
+    """
+    18 Jan 2008
+    Open a dircontrol to locate a folder, a text box of some kind to
+    show the progress, and build the segfonts file/list
+    """
+    def __init__( self, parent, startdir ):
+        wx.Dialog.__init__(self, parent, -1, _("Check for dangerous fonts."), \
+        size=(800,400),pos = wx.DefaultPosition, style=wx.DEFAULT_FRAME_STYLE )
+        
+        title = wx.StaticText( self,-1,  _("Choose a directory and double click it to start"))
+        self.output = wx.TextCtrl( self, -1, "...", \
+        size=(200,-1), style = wx.TE_MULTILINE | wx.TE_READONLY )
+        self.treedir = wx.GenericDirCtrl( self, -1, dir=startdir, style=wx.DIRCTRL_DIR_ONLY )
+        self.treedir.SetMinSize((200,200))
+        self.output.SetMinSize((200,200))
+        
+        sy = wx.BoxSizer(wx.VERTICAL)
+        sy.Add(title,0,wx.EXPAND|wx.ALL,border = 4)
+        sy.Add(self.output,1,wx.EXPAND | wx.ALL)
+
+        btnsizer = wx.StdDialogButtonSizer()
+        btn = wx.Button(self, wx.ID_OK)
+        btn.SetDefault()
+        btnsizer.AddButton(btn)
+        btnsizer.Realize()
+        
+        sy.Add(btn,0,wx.EXPAND)
+        
+        sz = wx.BoxSizer(wx.HORIZONTAL)
+        
+        sz.Add(self.treedir,1,wx.EXPAND)
+        sz.Add(sy,1, wx.EXPAND )# this wx.EXPAND got the text control to fit the height.
+
+        self.SetSizer(sz)
+
+        self.tree = self.treedir.GetTreeCtrl()
+
+        self.tree.Bind(wx.EVT_LEFT_DCLICK, self.__goFigure)
+
+    def printer( self, txt=None ):
+        if txt is None: txt = "\n"
+        self.output.WriteText(txt + "\n")
+
+    def __goFigure( self, e ):
+        self.output.Clear()
+        dirtocheck = self.treedir.GetPath()
+        fpsys.checkFonts( dirtocheck, self.printer )
+        
+        
+        
         
