@@ -27,7 +27,7 @@ Ramen.
 """
 
 import sys, os, locale
-import pathcontrol
+##import pathcontrol
 import strings
 import fpsys # Global objects
 import fontybugs
@@ -36,22 +36,18 @@ import fpversion
 
 ## Now, bring in all those big modules
 import wx
+import wx.lib.scrolledpanel
+import wx.lib.statbmp
 
 ## Setup wxPython to access translations : enables the stock buttons.
 langid = wx.LANGUAGE_DEFAULT # Picks this up from $LANG
 mylocale = wx.Locale( langid )
 
-##blah="blah"
-##print _("Test me %s") % blah
-
-import wx.lib.scrolledpanel
-import wx.lib.statbmp
-
 ## Fetch my own pubsub stuff
 from pubsub import *
 ps = CPubsub()
 
-## Fetch the dialogue classes *About and Settings*
+## Fetch the dialogue classes *About, Settings, Help, etc.*
 import dialogues
 
 ## Crappy flag to handle startup sizing issues.
@@ -81,7 +77,7 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
         self.bitmap = None
         self.prepareBitmap()
         
-        sz = (self.bitmap.GetWidth() , self.bitmap.GetHeight())
+        sz = (self.bitmap.GetWidth(), self.bitmap.GetHeight())
        
         ## Do mystical voodoo that I can't remember from 2006.
         self.gsb = wx.lib.statbmp.GenStaticBitmap.__init__(self,parent, -1, self.bitmap, pos, sz)
@@ -124,7 +120,7 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
             memDc.SetBackground( wx.Brush( block, wx.SOLID) )
             memDc.Clear()
             
-            self.__fadeEffect( memDc, self.minHeight, yll )
+            self.__fadeEffect( memDc, self.minHeight, self.width, yll )
             
             textTup = self.fitem.errorText()
             
@@ -157,13 +153,16 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
         ## after this loop.
         pilList=[]
         totheight = 0
-        for pilimage,height in self.fitem.generatePilFont( self.width, self.maxHeight ):
-            pilList.append( [pilimage, height] )
+        maxwidth = [self.width] # to figure-out the biggest width
+        for pilimage,height,pilwidth in self.fitem.generatePilFont( self.width, self.maxHeight ):
+            pilList.append( [pilimage, height, pilwidth] )
             totheight += height + spacer
+            maxwidth.append(pilwidth)
         ## Limit the minimum we allow.
         if totheight < self.minHeight:
             totheight = self.minHeight        
-            
+        maxwidth = max(maxwidth) # find it.
+        
         ## BADFONT cases
         ##  Badfonts are still available for installation, it's just that I can't
         ##  display their glyph or fam/style info (until PIL is patched).
@@ -190,9 +189,17 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
             ## Must highlight FILE_NOT_FOUND because it's not clickable at all.
             if self.fitem.badstyle == "FILE_NOT_FOUND":
                 block = (255,0,0)
-                fadecol = (160,0,0)
+                fadecol = (200,0,0)
                 fcol = "yellow"
                 bcol = fadecol                  
+            
+            ## Jan 18 2008
+            ## Let's make the segfault fonts obvious
+            if self.fitem.badstyle == "PIL_SEGFAULT_ERROR":
+                block = (255,255,0)
+                fadecol = (200,200,0)
+                fcol = "red"
+                bcol = fadecol 
                 
             wxMainBitmap = wx.EmptyImage( self.width, self.minHeight ).ConvertToBitmap()
             ## White it out
@@ -201,14 +208,14 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
             memDc.SetBackground(wx.Brush( block, wx.SOLID)) 
             memDc.Clear()
             ## Fade effect
-            self.__fadeEffect( memDc, self.minHeight, fadecol )
+            self.__fadeEffect( memDc, self.minHeight, self.width, fadecol )
 
             ## Message
             texty = 3
             self.__fatFont(memDc, 12, texty, fcol, bcol, 10, wx.NORMAL, self.fitem.badfontmsg )
             ## Path
             texty = 20
-            self.__fatFont(memDc, 12, texty, fcol, bcol, 7, wx.NORMAL, self.fitem.glyphpaf)
+            self.__fatFont(memDc, 12, texty, fcol, bcol, 7, wx.NORMAL, self.fitem.glyphpaf_unicode)
             ## Filename
             texty = 34
             self.__fatFont(memDc, 10, texty, fcol, bcol, 10, wx.NORMAL, "Name: %s" % self.name)
@@ -224,7 +231,7 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
         ## It's okay and normal and let's please f*c!#6 draw it already.
         else:
             ## Make one big bitmap to house one or more faces (subfaces)
-            wxMainBitmap = wx.EmptyImage( self.width, totheight ).ConvertToBitmap()
+            wxMainBitmap = wx.EmptyImage( maxwidth, totheight ).ConvertToBitmap()
             
             ## White it out
             memDc = wx.MemoryDC()
@@ -236,15 +243,15 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
             
             y = 0
             i = 0
-            for pilimage, glyphHeight in pilList:
-                wxfaceimage = wx.EmptyImage( self.width, glyphHeight )
+            for pilimage, glyphHeight, pilwidth in pilList:
+                wxfaceimage = wx.EmptyImage( pilwidth, glyphHeight )
                 try:
                 ## Get the data from PIL into wx
                     wxfaceimage.SetData( pilimage.convert('RGB').tostring() ) 
                     faceBitmap = wxfaceimage.ConvertToBitmap() 
                 except:
                     ps.pub( show_error_and_abort,\
-                    _("Error creating a wximage of %s") % self.fitem.glyphpaf)
+                    _("Error creating a wximage of %s") % self.fitem.glyphpaf_unicode)
                     
                 ## Place it into the main image, down a tad so it looks better.
                 facex = 10
@@ -255,7 +262,7 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
                 ## only if we are at the end of the faces
                 ## must be done here so text goes above it.
                 if i == self.fitem.numFaces-1:
-                    self.__fadeEffect( memDc, totheight, (218,218,218) )
+                    self.__fadeEffect( memDc, totheight, maxwidth, (218,218,218) )
                 
                 ## Now draw the text showing the font name/family/style.
                 ## -- suggested by Chris Mohler.
@@ -292,11 +299,12 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
         ## Now a dividing line
         memDc.SetPen( wx.Pen( lineCol, 2 ) ) 
         memDc.SetBrush( wx.TRANSPARENT_BRUSH ) 
-        memDc.DrawLine( 0, self.height-1, self.width, self.height-1 )
+        #memDc.DrawLine( 0, self.height-1, self.width, self.height-1 )
+        memDc.DrawLine( 0, self.height-1, maxwidth, self.height-1 )
 
         self.bitmap = wxMainBitmap
         
-    def __fadeEffect( self, memDc,liney, col ):
+    def __fadeEffect( self, memDc, liney, width, col ):
         """
         Baptiste's idea!
          "Now a dividing gradient, you can say "wow" ;-)"
@@ -313,7 +321,7 @@ class Fitmap(wx.lib.statbmp.GenStaticBitmap):
             b = min(255,int(1.5*q+sb))
             #print (r,g,b)
             memDc.SetPen( wx.Pen( wx.Colour( r, g ,b ), 1 ) ) 
-            memDc.DrawLine( 0,liney-q,self.width,liney-q )
+            memDc.DrawLine( 0, liney-q, width, liney-q )
 
     def __fatFont(self, dc, x, y, fcol, bcol, points, weight, txt):
         """
@@ -444,7 +452,10 @@ class ScrolledFontView(wx.lib.scrolledpanel.ScrolledPanel) :
             firstRunSizeFlag = True
             self.width = 1024 # I cannot get a sensible value for 1st run.
         else:
-            self.width = self.DoGetSize()[0] - sbwidth
+#TO DO : Jan 14 2008 - A user has reported an error on this command. Could it be 2.8 specific? 
+# I had used DoGetSize. The docs for 2.6 refer to base_DoGetSize. I have used that. On 2.8
+# I now get a deprecation warning...
+            self.width = self.base_DoGetSize()[0] - sbwidth
 
         ## Ensure we destroy all old fitmaps -- and I mean it.
         for f in self.fitmaps:
@@ -524,7 +535,7 @@ class FontViewPanel(wx.Panel):
         
         ## put them into the sizer
         sizerOtherControls.Add(self.textFilter, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
-        sizerOtherControls.Add( self.clearButton, 0, wx.ALIGN_LEFT ) # Clear button
+        sizerOtherControls.Add( self.clearButton, 0, wx.ALIGN_LEFT| wx.ALIGN_CENTER_VERTICAL ) # Clear button
         sizerOtherControls.Add(self.inputFilter, 1, wx.ALIGN_LEFT | wx.EXPAND)
         sizerOtherControls.Add(( 4,-1), 0, wx.EXPAND)
         sizerOtherControls.Add(self.choicePage, 0 ,wx.EXPAND | wx.ALIGN_RIGHT)  #Added it to the sizer
@@ -1064,7 +1075,7 @@ class TargetPogChooser(wx.Panel):
         self.buttPurge.Enable(not(installed))
         
     def SelectNoTargetPog(self, args):
-        ## Shadow
+        ## Shadow: this is dumb. I'm sorry. 
         self.__selectNoPog()
         
     def __selectNoPog(self):
@@ -1084,6 +1095,10 @@ class PogChooser(wx.ListCtrl) :
     """
     Basic list control for pogs - instanced by TargetPogChooser and NoteBook
     Parent: TargetPogChooser
+    
+    This single class (being used twice) causes terrible conceptual hardships
+    and forces all kinds of twisty tests. I'm sorry for this, we need a better
+    solution.
     """
     ## CLASS LEVEL variables - special things these.
     __poglistCopy = {} # To help in sorting.
@@ -1118,15 +1133,19 @@ class PogChooser(wx.ListCtrl) :
         
         self.AssignImageList(il, wx.IMAGE_LIST_SMALL) 
 
-        self.__PC = pathcontrol.PathControl()
+        self.__PC = fpsys.iPC # reuse the global pathcontrol object
         
         self.__fillPogList() 
         
+        ## Highlight the pog selected (the last one, or the cli chosen one)
         if select:
             i = self.FindItem(-1, select)
+            self.indexselected = i # Set this to help initial icon settings.
             self.Select(i, True)
         else:
             self.Select(0, False)
+            self.indexselected = -1
+
 
         self.ClearBackground() 
         self.items = None
@@ -1138,7 +1157,7 @@ class PogChooser(wx.ListCtrl) :
         
         #self.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
         
-        ## A subscribe line here, will register TWICE since this PogChooser object is instanced
+        ## This subscribe line here will register TWICE since this PogChooser object is instanced
         ## twice by the app...
         ps.sub(change_pog_icon, self.ChangeIcon)
         
@@ -1148,6 +1167,7 @@ class PogChooser(wx.ListCtrl) :
         so that we can compare them via locale.
         """
         s1,s2 = PogChooser.__poglistCopy[key1], PogChooser.__poglistCopy[key2]
+        ## Since the gui is unicode, I *think* I don't have to worry about errors here.
         return locale.strcoll( s1, s2 )
         
     def __onSelect(self, e):
@@ -1158,8 +1178,8 @@ class PogChooser(wx.ListCtrl) :
             ## We have clicked on the same Pog as last time - ergo do nothing.
             pognochange = True
         self.lastindexselected = self.indexselected # Record this for next time around
-        ##Raise a SELECT
-        textpogname = self.GetItemText(self.indexselected)
+
+        textpogname = self.GetItemText(self.indexselected) # Gets UNICODE !!!
         ## CALLS BOTH CLICK ROUTINES: onViewPogClick, OnTargetPogClick
         ## Each one decides where this comes from and rejects the other.
         ps.pub(pog_selected, textpogname, self.GetId(), pognochange)
@@ -1174,6 +1194,10 @@ class PogChooser(wx.ListCtrl) :
         Determines the images of the list items.
         Is called from TargetPogChooser as well (when clear button is clicked)
         """
+        
+        if pognochange is ():
+            pognochange = True
+            
         c = self.GetItemCount()
         sel = self.indexselected # the actual selection index, does not go to -1
         
@@ -1217,30 +1241,43 @@ class PogChooser(wx.ListCtrl) :
         """
         This is among the very FIRST things we do.
         Fill the list with pogs.
-        This will CULL any bad pogs (i.e. those with malformed files)
+        This will CULL any bad pogs (i.e. those with malformed content)
         Thus the PogInvalid error should not happen any more after a run.
         """
-        pl = self.__PC.getPogNames()
-        for p in pl :
+        #print "__fillPogList runs:\n\n"
+        pl = self.__PC.getPogNames() # pl will always be a BYTE STRING list
+        #print "pl:", pl
+        #print
+        
+        for p in pl: # 'p' is a byte string.
             ipog = fontcontrol.Pog(p)
             try: #catch pogs that are not properly formed
-                if ipog.isInstalled(): i = 1
+                if ipog.isInstalled(): i = 1 # isInstalled opens the pog file.
                 else: i = 0
-                #self.InsertImageStringItem(0, p ,0) 
-                li = wx.ListItem() 
-                li.SetImage(i) 
-                li.SetText(p) 
-                id = wx.NewId()
-                PogChooser.__poglistCopy[id] = p # record the pog name
-                row = self.InsertItem( li ) 
-                self.SetItemData( row, id ) # associate back to __poglistCopy
             except fontybugs.PogInvalid, eInst:
-                #raise
-                print _("(%s) skipped") % p
-            except:
-                #raise
-                print sys.exc_info()[0]
-                raise SystemExit
+                ## An "invalid" pog is one that does not have
+                ## installed/not installed on the first line.
+                print _(u"(%s) skipped. It's an invalid pog.") % [p]
+                continue
+
+            ## Let's try to make a unicode of p so li.SetText(p) can display it:
+            ## I think it's easier to do this myself.
+            try:
+                up = p.decode(locale.getpreferredencoding())
+                p = up
+            except UnicodeDecodeError:
+                ## We can't convert it under this locale
+                print _(u"(%s) skipped. I can't display this name under your locale.") % [p]
+                continue
+            ## Okay, we have a valid pog name to use:
+            li = wx.ListItem() 
+            li.SetImage(i) 
+            li.SetText(p) 
+            id = wx.NewId()
+            PogChooser.__poglistCopy[id] = p # record the pog name
+            row = self.InsertItem( li ) 
+            self.SetItemData( row, id ) # associate back to __poglistCopy
+
         self.SortItems( self.__Sorter )
         
     def AddItem(self, pogname):
@@ -1411,11 +1448,16 @@ class NoteBook(wx.Notebook):
             return
         ## Check pognochange, it means this is the same pog as last time.
         if args[2]: return 
-        try:
-            fpsys.instantiateViewPog(args[0])
-        except fontybugs.PogInvalid,e :
-            ps.pub(show_error_and_abort, unicode( e ))
-            return
+        
+        ## instantiateViewPog calls pog.genList which bubbles:
+        ## PogInvalid
+        ## BUT - this error only makes sense from the
+        ## cli pov. By the time the gui is running, that
+        ## pog has been renamed .badpog and therefore 
+        ## won't even appear in the list. So, don't bother
+        ## catching it.
+        fpsys.instantiateViewPog(args[0])
+
         if fpsys.state.samepogs: #forbid same pogs selection
             ps.pub(select_no_target_pog)
         else:
@@ -1449,6 +1491,7 @@ class StatusBar(wx.StatusBar):
 class MainFrame(wx.Frame):
     def __init__(self,parent,title) :
         ## Draw the frame
+        title = title + "   -   " + locale.getpreferredencoding()
         wx.Frame.__init__(self,parent,-1,title,fpsys.config.pos,fpsys.config.size) 
         
         ## Suddenly don't need this anymore.
@@ -1476,7 +1519,10 @@ class MainFrame(wx.Frame):
         menu1 = wx.Menu()
         menu1.Append(101, _("&Settings\tCtrl+S"), _("Change settings"))
         menu1.AppendSeparator()
-        self.exit = menu1.Append(102, _("&Close"), _("Close the app"))
+        ## Jan 18 2008
+        menu1.Append( 102, _("&Check fonts"), _("Find those fonts that crash Fonty.") )
+        
+        self.exit = menu1.Append(103, _("&Exit"), _("Close the app"))
         ## Add menu to the menu bar
         self.menuBar.Append(menu1, _("&File"))
 
@@ -1499,6 +1545,7 @@ class MainFrame(wx.Frame):
         
         self.Bind(wx.EVT_MENU, self.__onHandleESC, self.exit)
         self.Bind(wx.EVT_MENU, self.__menuSettings, id = 101)
+        self.Bind(wx.EVT_MENU, self.__menuCheckFonts, id = 102 )
         self.Bind(wx.EVT_MENU, self.__menuAbout, id = 202)
         self.Bind(wx.EVT_MENU, self.__menuHelp, id = 201)
 
@@ -1579,6 +1626,7 @@ class MainFrame(wx.Frame):
         ps.sub(uninstall_pog, self.UninstallPog)
         ps.sub(main_button_click, self.OnMainClick)
         
+        
         ## Dec 2007 - Actually this one does not function.
         ps.sub(menu_settings, self.__menuSettings)
         
@@ -1591,6 +1639,10 @@ class MainFrame(wx.Frame):
         ## call the big one - the big chief, the big cheese:
         self.UpdateFontViewPanel() #Go and fill in the font view and surrounding controls
         
+        ## A nasty looking line to call the SortOutTheDamnImages function
+        ## This is to draw the right icons dep on the params from cli.
+        self.panelTargetPogChooser.pogTargetlist.SortOutTheDamnImages(False)
+        
     def __onSize( self, evt ):
         """
         The splitter has been moved. Don't ask me why splitter and not splitter2
@@ -1731,7 +1783,7 @@ class MainFrame(wx.Frame):
                     to.write()      
                 except(fontybugs.PogWriteError), e:
                     bug = True
-                    self.errorBox([unicode( e ),])
+                    self.ErrorBox( [repr( e )] )
                 self.UpdateFontViewPanel()
                 if not bug:
                     self.sb.Report(_("Selected fonts are now in %s.") % to.label())
@@ -1787,6 +1839,22 @@ class MainFrame(wx.Frame):
         val = dlg.ShowModal()
         dlg.Destroy()
 
+    def __menuCheckFonts( self, e ):
+        """
+        Added Jan 18 2008
+        User can visit suspicious directories with this tool
+        to gather a list of fonts that kill the app. They will be
+        marked as such and hereafter be safe to use.
+        """
+        ## Set startdir to the one our own dircontrol is in
+        if fpsys.state.viewpattern == "F": 
+            startdir = fpsys.state.viewobject.path
+        else: 
+            ##Let's get it from the config object
+            startdir = fpsys.config.lastdir
+        dlg = dialogues.DialogCheckFonts( self, startdir )
+        val = dlg.ShowModal()
+        dlg.Destroy()
         
 ## Start the main frame and then show it.
 class App(wx.App) :
